dnl ---------------------------------------------------------------------------
dnl
dnl  adi.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2005-2007 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  process this file by using m4 to produce adi.c
dnl    m4 < adi.c.m4 > adi.c
dnl
dnl ---------------------------------------------------------------------------
include(`periodic.m4')dnl
/*---------------------------------------------------------------------------*/

/*!  \file  adi.c
     \brief  ADI subroutines for the field evolver.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from adi.c.m4.  */

#include "common.h"

#include "adi.h"

#define SQR(x) ((x) * (x))

/*---------------------------------------------------------------------------*/
void calculate_adi_source_1d(int nx,double dx2,
                             double h,double omega,
                             double *src,double *psi,
                             double *mu,double *mu_ext)
{
    int i;

    for(i = 0;i < nx;i++)
    {
        src[i] = h * psi[i];
        src[i] -= ((h * omega) * (mu[i]
                                  + mu_ext[i]));
    }
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_2d_x(int nx,int ny,
                               double dx2,double dy2,
                               double h,double omega,
                               double *src,double *psi,
                               double *mu,double *mu_ext)
{
    int i,j;
    int ip,im,jp,jm;

define(`OPERATION',dnl
            src[i * ny + j] =
                + h * psi[i * ny + j]
                + (dx2 / dy2) * (  psi[i * ny + jp]
                                 + psi[i * ny + jm]
                                 - 2 * psi[i * ny + j]);
            src[i * ny + j] -= ((omega * h / 2)
                                * (  mu[i * ny + j]
                                   + mu_ext[i * ny + j]));)dnl
PERIODIC2D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_2d_y(int nx,int ny,
                               double dx2,double dy2,
                               double h,double omega,
                               double *src,double *psi,
                               double *mu,double *mu_ext)
{
    int i,j;
    int ip,im,jp,jm;

define(`OPERATION',dnl
            src[i * ny + j] =
                + h * psi[i * ny + j]
                + (dy2 / dx2) * (  psi[ip * ny + j]
                                 + psi[im * ny + j]
                                 - 2 * psi[i * ny + j]);
            src[i * ny + j] -= ((omega * h / 2)
                                * (  mu[i * ny + j]
                                   + mu_ext[i * ny + j]));)dnl
PERIODIC2D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_3d_x(int nx,int ny,int nz,
                               double dx2,double dy2,double dz2,
                               double h,double omega,
                               double *src,double *psi,
                               double *mu,double *mu_ext)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;

define(`OPERATION',dnl
                src[(i * ny + j) * nz + k] =
                    + h * psi[(i * ny + j) * nz + k]
                    + (dx2 / dy2) * (  psi[(i * ny + jp) * nz + k]
                                     + psi[(i * ny + jm) * nz + k]
                                     - 2 * psi[(i * ny + j) * nz + k])
                    + (dx2 / dz2) * (  psi[(i * ny + j) * nz + kp]
                                     + psi[(i * ny + j) * nz + km]
                                     - 2 * psi[(i * ny + j) * nz + k]);
                src[(i * ny + j) * nz + k] -= ((omega * h / 3)
                                               * (  mu[(i * ny + j) * nz + k]
                                                  + mu_ext[(i * ny + j) * nz + k]));)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_3d_y(int nx,int ny,int nz,
                               double dx2,double dy2,double dz2,
                               double h,double omega,
                               double *src,double *psi,
                               double *mu,double *mu_ext)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;

define(`OPERATION',dnl
                src[(i * ny + j) * nz + k] =
                    + h * psi[(i * ny + j) * nz + k]
                    + (dy2 / dx2) * (  psi[(ip * ny + j) * nz + k]
                                     + psi[(im * ny + j) * nz + k]
                                     - 2 * psi[(i * ny + j) * nz + k])
                    + (dy2 / dz2) * (  psi[(i * ny + j) * nz + kp]
                                     + psi[(i * ny + j) * nz + km]
                                     - 2 * psi[(i * ny + j) * nz + k]);
                src[(i * ny + j) * nz + k] -= ((omega * h / 3)
                                               * (  mu[(i * ny + j) * nz + k]
                                                  + mu_ext[(i * ny + j) * nz + k]));)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_3d_z(int nx,int ny,int nz,
                               double dx2,double dy2,double dz2,
                               double h,double omega,
                               double *src,double *psi,
                               double *mu,double *mu_ext)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;

define(`OPERATION',dnl
                src[(i * ny + j) * nz + k] =
                    + h * psi[(i * ny + j) * nz + k]
                    + (dz2 / dx2) * (  psi[(ip * ny + j) * nz + k]
                                     + psi[(im * ny + j) * nz + k]
                                     - 2 * psi[(i * ny + j) * nz + k])
                    + (dz2 / dy2) * (  psi[(i * ny + jp) * nz + k]
                                     + psi[(i * ny + jm) * nz + k]
                                     - 2 * psi[(i * ny + j) * nz + k]);
                src[(i * ny + j) * nz + k] -= ((omega * h / 3)
                                               * (  mu[(i * ny + j) * nz + k]
                                                  + mu_ext[(i * ny + j) * nz + k]));)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_dynamics_1d(int nx,double dx2,
                                      double c,double omega,
                                      double *phi,
                                      double *psi,double *L_psi,
                                      double *mu_tot,double *L_mu_tot,
                                      double *xi)
{
    int i;
    int ip,im;

define(`OPERATION',dnl
        phi[i] +=
            + omega * (  psi[i] * L_mu_tot[i]
                       - mu_tot[i] * L_psi[i])
            + xi[i]
            - c / dx2 * (  SQR(psi[ip])
                         + SQR(psi[im])
                         - 2 * SQR(psi[i]));
)dnl
PERIODIC1D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_dynamics_2d_x(int nx,int ny,
                                        double dx2,
                                        double c,double omega,
                                        double *phi,
                                        double *psi,double *L_psi,
                                        double *mu_tot,double *L_mu_tot,
                                        double *xi)
{
    int i,j;
    int ip,im,jp,jm;

define(`OPERATION',dnl
            phi[i * ny + j] +=
                + omega * (  psi[i * ny + j] * L_mu_tot[i * ny + j]
                           - mu_tot[i * ny + j] * L_psi[i * ny + j])
                + xi[i * ny + j]
                - c / dx2 * (  SQR(psi[ip * ny + j])
                             + SQR(psi[im * ny + j])
                             - 2 * SQR(psi[i * ny + j]));
)dnl
PERIODIC2D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_dynamics_2d_y(int nx,int ny,
                                        double dy2,
                                        double c,double omega,
                                        double *phi,
                                        double *psi,double *L_psi,
                                        double *mu_tot,double *L_mu_tot,
                                        double *xi)
{
    int i,j;
    int ip,im,jp,jm;

define(`OPERATION',dnl
            phi[i * ny + j] +=
                + omega * (  psi[i * ny + j] * L_mu_tot[i * ny + j]
                           - mu_tot[i * ny + j] * L_psi[i * ny + j])
                + xi[i * ny + j]
                - c / dy2 * (  SQR(psi[i * ny + jp])
                             + SQR(psi[i * ny + jm])
                             - 2 * SQR(psi[i * ny + j]));
)dnl
PERIODIC2D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_dynamics_3d_x(int nx,int ny,int nz,
                                        double dx2,
                                        double c,double omega,
                                        double *phi,
                                        double *psi,double *L_psi,
                                        double *mu_tot,double *L_mu_tot,
                                        double *xi)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;

define(`OPERATION',dnl
                phi[(i * ny + j) * nz + k] +=
                    + omega * (  psi[(i * ny + j) * nz + k] * L_mu_tot[(i * ny + j) * nz + k]
                               - mu_tot[(i * ny + j) * nz + k] * L_psi[(i * ny + j) * nz + k])
                    + xi[(i * ny + j) * nz + k]
                    - c / dx2 * (  SQR(psi[(ip * ny + j) * nz + k])
                                 + SQR(psi[(im * ny + j) * nz + k])
                                 - 2 * SQR(psi[(i * ny + j) * nz + k]));
)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_dynamics_3d_y(int nx,int ny,int nz,
                                        double dy2,
                                        double c,double omega,
                                        double *phi,
                                        double *psi,double *L_psi,
                                        double *mu_tot,double *L_mu_tot,
                                        double *xi)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;

define(`OPERATION',dnl
                phi[(i * ny + j) * nz + k] +=
                    + omega * (  psi[(i * ny + j) * nz + k] * L_mu_tot[(i * ny + j) * nz + k]
                               - mu_tot[(i * ny + j) * nz + k] * L_psi[(i * ny + j) * nz + k])
                    + xi[(i * ny + j) * nz + k]
                    - c / dy2 * (  SQR(psi[(i * ny + jp) * nz + k])
                                 + SQR(psi[(i * ny + jm) * nz + k])
                                 - 2 * SQR(psi[(i * ny + j) * nz + k]));
)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
void calculate_adi_source_dynamics_3d_z(int nx,int ny,int nz,
                                        double dz2,
                                        double c,double omega,
                                        double *phi,
                                        double *psi,double *L_psi,
                                        double *mu_tot,double *L_mu_tot,
                                        double *xi)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;

define(`OPERATION',dnl
                phi[(i * ny + j) * nz + k] +=
                    + omega * (  psi[(i * ny + j) * nz + k] * L_mu_tot[(i * ny + j) * nz + k]
                               - mu_tot[(i * ny + j) * nz + k] * L_psi[(i * ny + j) * nz + k])
                    + xi[(i * ny + j) * nz + k]
                    - c / dz2 * (  SQR(psi[(i * ny + j) * nz + kp])
                                 + SQR(psi[(i * ny + j) * nz + km])
                                 - 2 * SQR(psi[(i * ny + j) * nz + k]));
)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/

dnl
dnl ---------------------------------------------------------------------------
