/*---------------------------------------------------------------------------*/

/*!  \file  blend.c
     \brief  some definitions of the polymer blends.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "blend.h"

/*---------------------------------------------------------------------------*/
polymer_blend *allocate_polymer_blend(int np,int *ns)
{
    int p,i,q;
    polymer_blend *blend;

    blend = xmalloc(sizeof(polymer_blend));

    blend->np = np;
    blend->ns = xmalloc(sizeof(int) * np);
    blend->volume_fraction = xmalloc(sizeof(double) * np);
    blend->chain = xmalloc(sizeof(polymer_chain *) * np);
    blend->propagator = xmalloc(sizeof(chain_propagator *) * np);
    blend->chi = xmalloc(sizeof(double ***) * np);

    for(p = 0;p < np;p++)
    {
        blend->ns[p] = ns[p];

        blend->chain[p] = allocate_polymer_chain(ns[p]);
        blend->propagator[p] = allocate_chain_propagator(ns[p]);
        blend->chi[p] = xmalloc(sizeof(double **) * ns[p]);

        for(i = 0;i < ns[p];i++)
        {
            blend->chi[p][i] = xmalloc(sizeof(double *) * np);
            for(q = 0;q < np;q++)
            {
                blend->chi[p][i][q] = xmalloc(sizeof(double) * ns[q]);
            }
        }
    }

    return blend;
}

/*---------------------------------------------------------------------------*/
void calculate_blend_propagator(polymer_blend *blend)
{
    int p;

    for(p = 0;p < blend->np;p++)
    {
        calculate_chain_propagator(blend->chain[p],blend->propagator[p]);
    }
}

/*---------------------------------------------------------------------------*/
void free_polymer_blend(polymer_blend *blend)
{
    int np,*ns;
    int p,i,q;

    np = blend->np;
    ns = blend->ns;

    if(blend == NULL)
        return;

    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            for(q = 0;q < np;q++)
            {
                free(blend->chi[p][i][q]);
            }
            free(blend->chi[p][i]);
        }

        free_polymer_chain(blend->chain[p]);
        free_chain_propagator(blend->propagator[p]);
        free(blend->chi[p]);
    }

    free(blend->ns);
    free(blend->volume_fraction);
    free(blend->chain);
    free(blend->propagator);
    free(blend->chi);

    free(blend);
}

/*---------------------------------------------------------------------------*/
int block_number(polymer_blend *blend)
{
    int p;
    int nc;

    nc = 0;
    for(p = 0;p < blend->np;p++)
        nc += blend->ns[p];

    return nc;
}

/*---------------------------------------------------------------------------*/
double psi_spatial_average(polymer_blend *blend,int p,int i)
{
    double psi_bar;

    psi_bar = sqrt(blend->volume_fraction[p] * blend->chain[p]->f[i]);

    return psi_bar;
}

/*---------------------------------------------------------------------------*/
