/*---------------------------------------------------------------------------*/

/*!  \file  blend.h
     \brief  some definitions of the polymer blends.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_BLEND_H
#define INCLUDE_BLEND_H

#include "polymer.h"
#include "propagator.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  polymer blend  */
typedef struct polymer_blend
{
        /*!  \brief  number of polymer species.  */
    int np;
        /*!  \brief  number of subchains.  */
    int *ns;
        /*!  \brief  average volume fraction.  */
    double *volume_fraction;
        /*!  \brief  subchain in polymers.  */
    polymer_chain **chain;
        /*!  \brief  propagator of polymers.  */
    chain_propagator **propagator;
        /*!  \brief  chi parameter (Flory-Huggins interaction parameter)  */
    double ****chi;
} polymer_blend;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new polymer blend
     \param  np  number of polymer species
     \param  ns  nubmer of subchains  */
polymer_blend *allocate_polymer_blend(int np,int *ns);
/*!  \brief  calculate propagator for polymers
     \param  blend  polymer blend  */
void calculate_blend_propagator(polymer_blend *blend);
/*!  \brief  free a polymer blend
     \param  blend  polymer blend to free  */
void free_polymer_blend(polymer_blend *blend);
/*!  \brief  returns a total number of subchains
     \param  blend  polymer blend  */
int block_number(polymer_blend *blend);
/*!  \brief  calculate the spatial average of psi field.
     \param  blend  polymer blend
     \param  p  index of polymer
     \param  i  index of subchain  */
double psi_spatial_average(polymer_blend *blend,int p,int i);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
