/*---------------------------------------------------------------------------*/

/*!  \file  condition.h
     \brief  simulation conditions.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005-2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_CONDITION_H
#define INCLUDE_CONDITION_H

/*---------------------------------------------------------------------------*/
/*!  \brief  simulation condition.  */
typedef struct simulation_condition
{
        /*!  \brief  whether we use the lattice optimization.  */
    int optimize_lattice;

        /*!  \brief  whether to use the multigrid Poisson equation solver.  */
    int use_multigrid_solver;

        /*!  \brief  whether to do the dynamics simulation.  */
    int dynamics_simulation;

        /*!  \brief  whether to save the data sequantially.  */
    int save_psi_sequential;
    int save_density_sequential;
    int save_chemical_potential_sequential;
    int save_free_energy_sequential;
    int save_geometry_sequential;
    int save_dx_sequential;

        /*!  \brief  allowed maximum error.  */
    double error_tolerance;
        /*!  \brief  seed for a random number generator.  */
    unsigned long seed;
        /*!  \brief  magnification of noise.  */
    double noise;
    double initial_noise;

        /*!  \brief  allowed minimum value of the density.  */
    double phi_min;

        /*!  \brief  maximum nubmer of iteration.  */
    int iteration_max;
        /*!  \brief  interval for writing data.  */
    int interval;
        /*!  \brief  acceleration factor.  */
    double omega;
} simulation_condition;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new simulation condition.  */
simulation_condition *allocate_simulation_condition(void);
/*!  \brief  free a simulation condition.
     \param  condition  simulation condition to free  */
void free_simulation_condition(simulation_condition *condition);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
