/*---------------------------------------------------------------------------*/

/*!  \file  evolver.h
     \brief  a field evolver for the statics.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_EVOLVER_H
#define INCLUDE_EVOLVER_H

#include "chemicalpotential.h"
#include "externalforce.h"
#include "mgpoisson.h"
#include "tridiag.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  evolves density field.
     \param  desity  density field to evolve
     \param  potential  chemical potential field
     \param  omega  acceleration factor  */
void evolve_density(density_field *density,
                    chemical_potential *potential,
                    double omega);
/*!  \brief  evolves density field by ADI method.
     \param  desity  density field to evolve
     \param  potential  chemical potential field
     \param  force  external force
     \param  noise  magnification of noize
     \param  plan  plan for fftw.
     \param  omega  acceleration factor
     \param  mgcondition  multigrid Poisson equation solver conditon
     \param  multigrid  multigrid field
     \param  use_multigrid  whether use multigrid Poisson equation solver
     \param  tridiag  tridiagonal matrix (work area for ADI)
     \param  u  work array for Sherman-Morrison formula  */
void evolve_density_adi(density_field *density,
                        chemical_potential *potential,
                        external_force *force,
                        double omega,double noise,
                        field_fftw_plan *plan,multigrid_condition *mgcondition,
                        multigrid_field *multigrid,int use_multigrid,
                        tridiag_matrix *tridiag,double *u);
/*!  \brief  evolves Lagrangian multiplier.
     \param  potential  chemical potential field
     \param  omega  acceleration factor  */
void evolve_multiplier(chemical_potential *potential,double omega);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
