/*---------------------------------------------------------------------------*/

/*!  \file  externalforce.c
     \brief  external force field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "externalforce.h"

/*---------------------------------------------------------------------------*/
external_force *allocate_external_force(polymer_blend *blend,
                                        field_geometry *geometry)
{
    int np,*ns;
    int n;
    int p,i;
    external_force *force;

    np = blend->np;
    ns = blend->ns;
    n = lattice_point_number(geometry);

    force = xmalloc(sizeof(external_force));

    force->blend = blend;
    force->geometry = geometry;
    force->F = xmalloc(sizeof(double **) * np);

    for(p = 0;p < np;p++)
    {
        force->blend->ns[p] = ns[p];
        force->F[p] = xmalloc(sizeof(double *) * ns[p]);

        for(i = 0;i < ns[p];i++)
        {
            force->F[p][i] = xmalloc(sizeof(double) * n);
        }
    }

    return force;
}

/*---------------------------------------------------------------------------*/
void free_external_force(external_force *force)
{
    int np,*ns;
    int p,i;

    if(force == NULL)
        return;

    np = force->blend->np;
    ns = force->blend->ns;

    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            free(force->F[p][i]);
        }

        free(force->F[p]);
    }

    free(force->F);

    free(force);
}

/*---------------------------------------------------------------------------*/
void add_force_to_chemical_potential(chemical_potential *potential,
                                     external_force *force)
{
    int p,i,r;
    int np,*ns;
    int n;
    double ***mu_ext;
    double ***F;

    if(force == NULL)
        return;

    np = potential->blend->np;
    ns = potential->blend->ns;
    n = lattice_point_number(potential->geometry);
    F = force->F;
    mu_ext = potential->mu_ext;

    for(p = 0;p < np;p++)
    {
        for(i = 0;i < ns[p];i++)
        {
            for(r = 0;r < n;r++)
            {
                mu_ext[p][i][r] += F[p][i][r];
            }
        }
    }

}

/*---------------------------------------------------------------------------*/
