/*---------------------------------------------------------------------------*/

/*!  \file  externalforce.h
     \brief  external force field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_EXTERNALFORCE_H
#define INCLUDE_EXTERNALFORCE_H

#include "blend.h"
#include "geometry.h"
#include "chemicalpotential.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  external force field.  */
typedef struct external_force
{
        /*!  \brief  polymer blend.  */
    polymer_blend *blend;
        /*!  \brief  geometry for the field.  */
    field_geometry *geometry;

        /*!  \brief  external force.  */
    double ***F;
} external_force;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new external force.
     \param  blend  polymer blend
     \param  geometry geometry for the field  */
external_force *allocate_external_force(polymer_blend *blend,
                                        field_geometry *geometry);
/*!  \brief  free a external force.
     \param  force  external force field to free  */
void free_external_force(external_force *force);

/*!  \brief  add external force to chemical potential.
     \param  potential  chemical potential field
     \param  force  external force  */
void add_force_to_chemical_potential(chemical_potential *potential,
                                     external_force *force);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
