/*---------------------------------------------------------------------------*/

/*!  \file  freeenergy.h
     \brief  calculate the free energy of the system.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_FREEENERGY_H
#define INCLUDE_FREEENERGY_H

#include "density.h"
#include "operator.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  calculate the free energy of the system.
     \param  density  density field
     \param  plan  plan for FFT  */
double calculate_free_energy(density_field *density,
                             field_fftw_plan *plan);
/*!  \brief  calculate the long range term in the free energy.
     \param  density  density field
     \param  plan  plan for FFT  */
double calculate_free_energy_longrange(density_field *density,
                                       field_fftw_plan *plan);
/*!  \brief  calculate the local term in the free energy.
     \param  density  density field  */
double calculate_free_energy_local(density_field *density);
/*!  \brief  calculate the gradient term in the free energy.
     \param  density  density field
     \param  plan  plan for FFT  */
double calculate_free_energy_gradient(density_field *density,
                                      field_fftw_plan *plan);
/*!  \brief  calculate the free energy for homogeneous state.
     \param  density  density field  */
double calculate_free_energy_homogeneous(polymer_blend *blend);

/*!  \brief  calculate the differential of the free energy.
     \param  density  density field
     \param  plan  plan for FFT  */
double calculate_free_energy_differential(density_field *density,
                                          field_fftw_plan *plan);

/*!  \brief  calculate the differential of the free energy.
     \param  density  density field
     \param  plan  plan for FFT  */
double calculate_free_energy_differential2(density_field *density,
                                          field_fftw_plan *plan);

/*!  \brief  calculate estimated optimal lattice size.
     \param  density  density field
     \param  plan  plan for FFT  */
double calculate_optimal_lattice_size(density_field *density,
                                      field_fftw_plan *plan);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
