/*---------------------------------------------------------------------------*/

/*!  \file  gaussseidel.c
     \brief  Gauss-Seidel relaxation library.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from gaussseidel.c.m4.  */

#include "common.h"

#include "gaussseidel.h"

/*---------------------------------------------------------------------------*/
void gauss_seidel_1d(int nx,double lx,
                     double *u,double *f)
{
    int i;
    int ip,im;
    double hx2;

    hx2 = (lx / nx) * (lx / nx);

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        u[i] = ((u[ip] + u[im]) + hx2 * f[i]) / 2;
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        u[i] = ((u[ip] + u[im]) + hx2 * f[i]) / 2;
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        u[i] = ((u[ip] + u[im]) + hx2 * f[i]) / 2;
    }
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_2d(int nx,int ny,
                     double lx,double ly,
                     double *u,double *f)
{
    int i,j;
    int ip,im,jp,jm;
    double hx2,hy2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));
        }
    }
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_3d(int nx,int ny,int nz,
                     double lx,double ly,double lz,
                     double *u,double *f)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;
    double hx2,hy2,hz2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);
    hz2 = (lz / nz) * (lz / nz);

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_iteration_1d(int n_iteration,
                               int nx,double lx,
                               double *u,double *f)
{
    int i;

    for(i = 0;i < n_iteration;i++)
        gauss_seidel_1d(nx,lx,u,f);
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_iteration_2d(int n_iteration,
                               int nx,int ny,
                               double lx,double ly,
                               double *u,double *f)
{
    int i;

    for(i = 0;i < n_iteration;i++)
        gauss_seidel_2d(nx,ny,lx,ly,u,f);
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_iteration_3d(int n_iteration,
                               int nx,int ny,int nz,
                               double lx,double ly,double lz,
                               double *u,double *f)
{
    int i;

    for(i = 0;i < n_iteration;i++)
        gauss_seidel_3d(nx,ny,nz,lx,ly,lz,u,f);
}

/*---------------------------------------------------------------------------*/
double calculate_residual_1d(int nx,double lx,
                             double *u,double *f,double *r)
{
    int i;
    int ip,im;
    double norm2;
    double hx2;

    hx2 = (lx / nx) * (lx / nx);

    norm2 = 0;

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        r[i] = (u[ip] + u[im] - 2 * u[i]) / hx2 + f[i];
        norm2 += r[i] * r[i];
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        r[i] = (u[ip] + u[im] - 2 * u[i]) / hx2 + f[i];
        norm2 += r[i] * r[i];
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        r[i] = (u[ip] + u[im] - 2 * u[i]) / hx2 + f[i];
        norm2 += r[i] * r[i];
    }

    return norm2 / nx;
}

/*---------------------------------------------------------------------------*/
double calculate_residual_2d(int nx,int ny,
                             double lx,double ly,
                             double *u,double *f,double *r)
{
    int i,j;
    int ip,im,jp,jm;
    double norm2;
    double hx2,hy2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);

    norm2 = 0;

    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];
        }
    }

    return norm2 / nx / ny;
}

/*---------------------------------------------------------------------------*/
double calculate_residual_3d(int nx,int ny,int nz,
                             double lx,double ly,double lz,
                             double *u,double *f,double *r)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;
    double norm2;
    double hx2,hy2,hz2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);
    hz2 = (lz / nz) * (lz / nz);

    norm2 = 0;


    for(i = 0;i < 1;i++)
    {
        im = nx - 1;
        ip = 1 % nx;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
    }
    for(;i < (nx - 1);i++)
    {
        im = i - 1;
        ip = i + 1;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
    }
    for(;i < nx;i++)
    {
        im = nx - 2;
        ip = 0;
        for(j = 0;j < 1;j++)
        {
            jm = ny - 1;
            jp = 1 % ny;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
        for(;j < (ny - 1);j++)
        {
            jm = j - 1;
            jp = j + 1;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
        for(;j < ny;j++)
        {
            jm = ny - 2;
            jp = 0;
            for(k = 0;k < 1;k++)
            {
                km = nz - 1;
                kp = 1 % nz;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < (nz - 1);k++)
            {
                km = k - 1;
                kp = k + 1;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
            for(;k < nz;k++)
            {
                km = nz - 2;
                kp = 0;
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];
            }
        }
    }

    return norm2 / nx / ny / nz;
}

/*---------------------------------------------------------------------------*/
