dnl ---------------------------------------------------------------------------
dnl
dnl  gaussseidel.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2005 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  process this file by using m4 to produce gaussseidel.c
dnl    m4 < gaussseidel.c.m4 > gaussseidel.c
dnl
dnl ---------------------------------------------------------------------------
include(`periodic.m4')dnl
/*---------------------------------------------------------------------------*/

/*!  \file  gaussseidel.c
     \brief  Gauss-Seidel relaxation library.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from gaussseidel.c.m4.  */

#include "common.h"

#include "gaussseidel.h"

/*---------------------------------------------------------------------------*/
void gauss_seidel_1d(int nx,double lx,
                     double *u,double *f)
{
    int i;
    int ip,im;
    double hx2;

    hx2 = (lx / nx) * (lx / nx);

define(`OPERATION',dnl
        u[i] = ((u[ip] + u[im]) + hx2 * f[i]) / 2;)dnl
PERIODIC1D()
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_2d(int nx,int ny,
                     double lx,double ly,
                     double *u,double *f)
{
    int i,j;
    int ip,im,jp,jm;
    double hx2,hy2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);

define(`OPERATION',dnl
            u[i * ny + j] =
                (  hy2 * (u[ip * ny + j] + u[im * ny + j])
                 + hx2 * (u[i * ny + jp] + u[i * ny + jm])
                 + hx2 * hy2 * f[i * ny + j])
                / (2 * (hx2 + hy2));)dnl
PERIODIC2D()
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_3d(int nx,int ny,int nz,
                     double lx,double ly,double lz,
                     double *u,double *f)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;
    double hx2,hy2,hz2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);
    hz2 = (lz / nz) * (lz / nz);

define(`OPERATION',dnl
                u[(i * ny + j) * nz + k] =
                    (  hy2 * hz2 * (u[(ip * ny + j) * nz + k]
                                    + u[(im * ny + j) * nz + k])
                     + hz2 * hx2 * (u[(i * ny + jp) * nz + k]
                                    + u[(i * ny + jm) * nz + k])
                     + hx2 * hy2 * (u[(i * ny + j) * nz + kp]
                                    + u[(i * ny + j) * nz + km])
                     + hx2 * hy2 * hz2 * f[(i * ny + j) * nz + k])
                    / (2 * (hx2 * hy2 + hy2 * hz2 + hz2 * hx2));)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_iteration_1d(int n_iteration,
                               int nx,double lx,
                               double *u,double *f)
{
    int i;

    for(i = 0;i < n_iteration;i++)
        gauss_seidel_1d(nx,lx,u,f);
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_iteration_2d(int n_iteration,
                               int nx,int ny,
                               double lx,double ly,
                               double *u,double *f)
{
    int i;

    for(i = 0;i < n_iteration;i++)
        gauss_seidel_2d(nx,ny,lx,ly,u,f);
}

/*---------------------------------------------------------------------------*/
void gauss_seidel_iteration_3d(int n_iteration,
                               int nx,int ny,int nz,
                               double lx,double ly,double lz,
                               double *u,double *f)
{
    int i;

    for(i = 0;i < n_iteration;i++)
        gauss_seidel_3d(nx,ny,nz,lx,ly,lz,u,f);
}

/*---------------------------------------------------------------------------*/
double calculate_residual_1d(int nx,double lx,
                             double *u,double *f,double *r)
{
    int i;
    int ip,im;
    double norm2;
    double hx2;

    hx2 = (lx / nx) * (lx / nx);

    norm2 = 0;

define(`OPERATION',dnl
        r[i] = (u[ip] + u[im] - 2 * u[i]) / hx2 + f[i];
        norm2 += r[i] * r[i];)dnl
PERIODIC1D()

    return norm2 / nx;
}

/*---------------------------------------------------------------------------*/
double calculate_residual_2d(int nx,int ny,
                             double lx,double ly,
                             double *u,double *f,double *r)
{
    int i,j;
    int ip,im,jp,jm;
    double norm2;
    double hx2,hy2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);

    norm2 = 0;

define(`OPERATION',dnl
            r[i * ny + j] =
                  (u[ip * ny + j] + u[im * ny + j] - 2 * u[i * ny + j]) / hx2
                + (u[i * ny + jp] + u[i * ny + jm] - 2 * u[i * ny + j]) / hy2
                + f[i * ny + j];
            norm2 += r[i * ny + j] * r[i * ny + j];)dnl
PERIODIC2D()

    return norm2 / nx / ny;
}

/*---------------------------------------------------------------------------*/
double calculate_residual_3d(int nx,int ny,int nz,
                             double lx,double ly,double lz,
                             double *u,double *f,double *r)
{
    int i,j,k;
    int ip,im,jp,jm,kp,km;
    double norm2;
    double hx2,hy2,hz2;

    hx2 = (lx / nx) * (lx / nx);
    hy2 = (ly / ny) * (ly / ny);
    hz2 = (lz / nz) * (lz / nz);

    norm2 = 0;


define(`OPERATION',dnl
                r[(i * ny + j) * nz + k] =
                    + (u[(ip * ny + j) * nz + k] + u[(im * ny + j) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hx2
                    + (u[(i * ny + jp) * nz + k] + u[(i * ny + jm) * nz + k]
                       - 2 * u[(i * ny + j) * nz + k]) / hy2
                    + (u[(i * ny + j) * nz + kp] + u[(i * ny + j) * nz + km]
                       - 2 * u[(i * ny + j) * nz + k]) / hz2
                    + f[(i * ny + j) * nz + k];
                norm2 += r[(i * ny + j) * nz + k] * r[(i * ny + j) * nz + k];)dnl
PERIODIC3D()

    return norm2 / nx / ny / nz;
}

/*---------------------------------------------------------------------------*/
dnl
dnl ---------------------------------------------------------------------------
