/*---------------------------------------------------------------------------*/

/*!  \file  geometry.h
     \brief  the geometry infomation for the field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_GEOMETRY_H
#define INCLUDE_GEOMETRY_H

#ifdef HAVE_FFTW3_H
#include <fftw3.h>
#endif

/*---------------------------------------------------------------------------*/
/*!  \brief  geometry for a field  */
typedef struct field_geometry
{
        /*!  \brief  dimension of a field.  */
    int dimension;
        /*!  \brief  number of division.  */
    int nx,ny,nz;
        /*!  \brief  length of a edge.  */
    double lx,ly,lz;

        /*!  \brief q^2.  */
    double *q2;
        /*!  \brief 1 / q^2.  */
    double *q2i;
} field_geometry;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new field geometry.  */
field_geometry *allocate_field_geometry(void);
/*!  \brief  free a field geometry.
     \param  geometry  field geometry to free  */
void free_field_geometry(field_geometry *geometry);
/*!  \brief  returns a nubmer of lattice points.
     \param  geometry  geometry of a field  */
int lattice_point_number(field_geometry *geometry);
/*!  \brief  returns a nubmer of lattice points for a complex field.
     \param  geometry  geometry of a field  */
int lattice_point_number_complex(field_geometry *geometry);

/*!  \brief  allocate q^2 and 1 / q^2.
     \param  geometry  geometry of a field  */
void allocate_q2_q2i(field_geometry *geometry);
/*!  \brief  free q^2 and 1 / q^2.
     \param  geometry  geometry of a field  */
void free_q2_q2i(field_geometry *geometry);

/*!  \brief  calculate q^2.
     \param  geometry  geometry of a field  */
void calculate_q2(field_geometry *geometry);
/*!  \brief  calculate 1 / q^2.
     \param  geometry  geometry of a field  */
void calculate_q2i(field_geometry *geometry);

/*!  \brief  print the lattice size to \a fp.
     \param  fp  file pointer
     \param  geometry  geometry of a field  */
void print_lattice(FILE *fp,field_geometry *geometry);

/*!  \brief  calculate the maximum number of division, max(\a nx, \a ny, \a nz).
     \param  geometry  geometry of a field  */
int maximum_number_of_division_in_one_axis(field_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
