dnl ---------------------------------------------------------------------------
dnl
dnl  laplacian.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2005 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  process this file by using m4 to produce laplacian.c
dnl    m4 < laplacian.c.m4 > laplacian.c
dnl
dnl ---------------------------------------------------------------------------
include(`periodic.m4')dnl
/*---------------------------------------------------------------------------*/

/*!  \file  laplacian.c
     \brief  Laplacian operator for fields.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from laplacian.c.m4.  */

#include "common.h"

#include "laplacian.h"

/*---------------------------------------------------------------------------*/
void operate_L_1d(field_geometry *geometry,double *psi,double *L_psi)
{
    int i;
    int nx;
    double lx,dx,dx2;
    int ip,im;

    nx = geometry->nx;
    lx = geometry->lx;
    dx = lx / nx;
    dx2 = dx * dx;

define(`OPERATION',dnl
        L_psi[i] = (psi[im] + psi[ip] - 2 * psi[i]) / dx2;)dnl
PERIODIC1D()
}

/*---------------------------------------------------------------------------*/
void operate_L_2d(field_geometry *geometry,double *psi,double *L_psi)
{
    int i,j,nx,ny;
    int ip,im,jp,jm;
    double lx,ly,dx,dy,dx2,dy2;

    nx = geometry->nx;
    ny = geometry->ny;
    lx = geometry->lx;
    ly = geometry->ly;
    dx = lx / nx;
    dy = ly / ny;
    dx2 = dx * dx;
    dy2 = dy * dy;

define(`OPERATION',dnl
            L_psi[i * ny + j] =
                  (psi[ip * ny + j] + psi[im * ny + j]
                   - 2 * psi[i * ny + j]) / dx2
                + (psi[i * ny + jp] + psi[i * ny + jm]
                   - 2 * psi[i * ny + j]) / dy2;)dnl
PERIODIC2D()
}

/*---------------------------------------------------------------------------*/
void operate_L_3d(field_geometry *geometry,double *psi,double *L_psi)
{
    int i,j,k;
    int nx,ny,nz;
    int ip,im,jp,jm,kp,km;
    double lx,ly,lz,dx,dy,dz,dx2,dy2,dz2;

    nx = geometry->nx;
    ny = geometry->ny;
    nz = geometry->nz;
    lx = geometry->lx;
    ly = geometry->ly;
    lz = geometry->lz;
    dx = lx / nx;
    dy = ly / ny;
    dz = lz / nz;
    dx2 = dx * dx;
    dy2 = dy * dy;
    dz2 = dz * dz;

define(`OPERATION',dnl
                L_psi[(i * ny + j) * nz + k] =
                      (psi[(ip * ny + j) * nz + k]
                       + psi[(im * ny + j) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dx2
                    + (psi[(i * ny + jp) * nz + k]
                       + psi[(i * ny + jm) * nz + k]
                       - 2 * psi[(i * ny + j) * nz + k]) / dy2
                    + (psi[(i * ny + j) * nz + kp]
                       + psi[(i * ny + j) * nz + km]
                       - 2 * psi[(i * ny + j) * nz + k]) / dz2;)dnl
PERIODIC3D()
}

/*---------------------------------------------------------------------------*/
dnl
dnl ---------------------------------------------------------------------------
