/*---------------------------------------------------------------------------*/

/*!  \file  luaread.h
     \brief  read input Lua file for the simulator  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_LUAREAD_H
#define INCLUDE_LUAREAD_H

#include "condition.h"
#include "iofiles.h"
#include "blend.h"
#include "geometry.h"
#include "mgpoisson.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  read data from input Lua file.
     \param  input_lua  input Lua script file
     \param  condition  simulation condition
     \param  iofiles  simulation input / output files
     \param  geometry  geometry for the field
     \param  blend  polymer blend
     \param  mgcondition  multigrid Poisson solver condition  */
void read_input_lua(char *input_lua,
                    simulation_condition **condition,
                    simulation_iofiles **iofiles,
                    field_geometry **geometry,
                    polymer_blend **blend,
                    multigrid_condition **mgcondition);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
