/*---------------------------------------------------------------------------*/

/*!  \file  mgpoisson.h
     \brief  the multigrid Poisson equation solver.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_MGPOISSON_H
#define INCLUDE_MGPOISSON_H

#include "multigrid.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  condition for multigrid Poisson equation solver.  */
typedef struct multigrid_condition
{
        /*!  \brief  number of iterations for multigrid V-cycle.  */
    int n_cycle;
        /*!  \brief  number of iterations for pre-smoothing.  */
    int n_pre;
        /*!  \brief  number of iterations for post-smoothing.  */
    int n_post;
        /*!  \brief  error trenance for the mean square residual.  */
    double error_tolerance;
} multigrid_condition;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new multigird condition. */
multigrid_condition *allocate_multigrid_condition(void);
/*!  \brief  free a multigrid condition.
     \param  mgcondition  multigrid condition to free  */
void free_multigrid_condition(multigrid_condition *mgcondition);

/*!  \brief  operate Laplacian inverse to a field.
     \param  multigrid  multigrid field for the solver
     \param  mgcondition  multigrid condition for the solver
     \param  psi  field
     \param  Li_psi Laplacian inverse of \a psi  */
int operate_Li_multigrid(multigrid_field *multigrid,
                         multigrid_condition *mgcondition,
                         double *psi,double *Li_psi);

/*---------------------------------------------------------------------------*/
#endif
