dnl ---------------------------------------------------------------------------
dnl
dnl  noise.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2005,2006 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  process this file by using m4 to produce noise.c
dnl    m4 < noise.c.m4 > noise.c
dnl
dnl ---------------------------------------------------------------------------
include(`periodic.m4')dnl
/*---------------------------------------------------------------------------*/

/*!  \file  noise.c
     \brief  calculate the noise using fluctuation-dissipation relation.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from noise.c.m4.  */

#include "common.h"

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "nrand.h"
#include "noise.h"

/*---------------------------------------------------------------------------*/
static void clear_field_noise(field_geometry *geometry,double *xi);
static void calculate_field_noise_dynamics_1d(field_geometry *geometry,
                                              double *psi,double *xi,
                                              double omega);
static void calculate_field_noise_dynamics_2d(field_geometry *geometry,
                                              double *psi,double *xi,
                                              double omega);
static void calculate_field_noise_dynamics_3d(field_geometry *geometry,
                                              double *psi,double *xi,
                                              double omega);

/*---------------------------------------------------------------------------*/
void calculate_field_noise_dynamics(density_field *density,double omega)
{
    int p,i;
    double ***psi;
    double ***xi;
    field_geometry *geometry;

    psi = density->psi;
    xi = density->xi;
    geometry = density->geometry;

    for(p = 0;p < density->blend->np;p++)
    {
        for(i = 0;i < density->blend->ns[p];i++)
        {
            switch(geometry->dimension)
            {
                case 1:
                    calculate_field_noise_dynamics_1d(geometry,psi[p][i],
                                                      xi[p][i],omega);
                    break;

                case 2:
                    calculate_field_noise_dynamics_2d(geometry,psi[p][i],
                                                      xi[p][i],omega);
                    break;

                case 3:
                    calculate_field_noise_dynamics_3d(geometry,psi[p][i],
                                                      xi[p][i],omega);
                    break;

                default:
                    break;
            }
        }
    }
}

/*---------------------------------------------------------------------------*/
void clear_field_noise(field_geometry *geometry,double *xi)
{
    int r;

    for(r = 0;r < lattice_point_number(geometry);r++)
        xi[r] = 0;
}

/*---------------------------------------------------------------------------*/
void calculate_field_noise_dynamics_1d(field_geometry *geometry,double *psi,
                                       double *xi,double omega)
{
    int i,nx;
    int ip;
    double lx,dx;
    double cx;

    nx = geometry->nx;
    lx = geometry->lx;
    dx = lx / nx;

    clear_field_noise(geometry,xi);

define(`OPERATION',dnl
        cx = sqrt(omega * (psi[ip] * psi[i]) / dx) * nrand();

        xi[i] += cx / dx;
        xi[ip] -= cx / dx;)dnl
PERIODIC1DHALF()
}

/*---------------------------------------------------------------------------*/
void calculate_field_noise_dynamics_2d(field_geometry *geometry,double *psi,
                                       double *xi,double omega)
{
    int i,j,nx,ny;
    int ip,jp;
    double lx,ly,dx,dy;
    double cx,cy;

    nx = geometry->nx;
    ny = geometry->ny;
    lx = geometry->lx;
    ly = geometry->ly;
    dx = lx / nx;
    dy = ly / ny;

    clear_field_noise(geometry,xi);

define(`OPERATION',dnl
            cx = sqrt(omega * (psi[ip * ny + j] * psi[i * ny + j]) / dx / dy) * nrand();
            cy = sqrt(omega * (psi[i * ny + jp] * psi[i * ny + j]) / dx / dy) * nrand();

            xi[i * ny + j] += cx / dx;
            xi[ip * ny + j] -= cx / dx;
            xi[i * ny + j] += cy / dy;
            xi[i * ny + jp] -= cy / dy;)dnl
PERIODIC2DHALF()
}

/*---------------------------------------------------------------------------*/
void calculate_field_noise_dynamics_3d(field_geometry *geometry,double *psi,
                                       double *xi,double omega)
{
    int i,j,k,nx,ny,nz;
    int ip,jp,kp;
    double lx,ly,lz,dx,dy,dz;
    double cx,cy,cz;

    nx = geometry->nx;
    ny = geometry->ny;
    nz = geometry->nz;
    lx = geometry->lx;
    ly = geometry->ly;
    lz = geometry->lz;
    dx = lx / nx;
    dy = ly / ny;
    dz = lz / nz;

    clear_field_noise(geometry,xi);

define(`OPERATION',dnl
                cx = sqrt(omega * (psi[(ip * ny + j) * nz + k] 
                                   * psi[(i * ny + j) * nz + k]) / dx / dy / dz) * nrand();
                cy = sqrt(omega * (psi[(i * ny + jp) * nz + k]
                                   * psi[(i * ny + j) * nz + k]) / dx / dy / dz) * nrand();
                cz = sqrt(omega * (psi[(i * ny + j) * nz + kp] 
                                   * psi[(i * ny + j) * nz + k]) / dx / dy / dz) * nrand();

                xi[(i * ny + j) * nz + k] += cx / dx;
                xi[(ip * ny + j) * nz + k] -= cx / dx;
                xi[(i * ny + j) * nz + k] += cy / dy;
                xi[(i * ny + jp) * nz + k] -= cy / dy;
                xi[(i * ny + j) * nz + k] += cz / dz;
                xi[(i * ny + j) * nz + kp] -= cz / dz;)dnl
PERIODIC3DHALF()
}

/*---------------------------------------------------------------------------*/
