/*---------------------------------------------------------------------------*/

/*!  \file  polymer.c
     \brief  some definitions of structures for polymers.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2005,2006 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "xmalloc.h"
#include "matrix.h"
#include "polymer.h"

/*---------------------------------------------------------------------------*/
polymer_chain *allocate_polymer_chain(int n)
{
    polymer_chain *chain;

    chain = xmalloc(sizeof(polymer_chain));

    chain->ns = n;
    chain->f = allocate_vector(n);
    chain->b = allocate_vector(n);
    chain->l2 = allocate_matrix(n);
    chain->monomer = xmalloc(sizeof(int) * n);

    return chain;
}

/*---------------------------------------------------------------------------*/
void free_polymer_chain(polymer_chain *chain)
{
    if(chain == NULL)
        return;

    free_vector(chain->f);
    free_vector(chain->b);
    free_matrix(chain->l2);
    free(chain->monomer);

    free(chain);
}

/*---------------------------------------------------------------------------*/
