/*---------------------------------------------------------------------------*/

/*!  \file  field.h
     \brief  the polymer density / chemical potential field information.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_FIELD_H
#define INCLUDE_FIELD_H

#include "geometry.h"

/*---------------------------------------------------------------------------*/
/*!  \biref  allocate a new field (double precision).
     \param  geometry  geometry of the field
     \param  n  number of components  */
double **allocate_field_double(field_geometry *geometry,int n);
/*!  \biref  free a field (double precision).
     \param  u  field to free  */
void free_field_double(double **u);
/*!  \biref  allocate a new field (single precision).
     \param  geometry  geometry of the field
     \param  n  number of components  */
float **allocate_field_float(field_geometry *geometry,int n);
/*!  \biref  free a field (single precision).
     \param  u  field to free  */
void free_field_float(float **u);

/*!  \biref  convert a double precisioin field into single precision field.
     \param  u  field to convert  */
void convert_field_double_to_float(double **u_double,float **u_float,
                                   field_geometry *geometry,int n);
/*!  \biref  convert a single precisioin field into double precision field.
     \param  u  field to convert  */
void convert_field_float_to_double(float **u_float,double **u_double,
                                   field_geometry *geometry,int n);
/*!  \biref  convert a field into its square root (density into psi).
     \param  u  field to convert  */
void convert_field_sqrt_float(float **u,field_geometry *geometry,int n);
/*!  \biref  convert a field into its square root (density into psi).
     \param  u  field to convert  */
void convert_field_sqrt_double(double **u,field_geometry *geometry,int n);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
