/*---------------------------------------------------------------------------*/

/*!  \file  geometry.h
     \brief  the geometry infomation for the field.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_GEOMETRY_H
#define INCLUDE_GEOMETRY_H

/*---------------------------------------------------------------------------*/
/*!  \brief  geometry for density / chemical potential fields  */
typedef struct field_geometry
{
        /*!  \brief  dimension of a field.  */
    int dimension;
        /*!  \brief  number of division.  */
    int n[3];
        /*!  \brief  length of a edge.  */
    double l[3];
} field_geometry;

/*---------------------------------------------------------------------------*/
/*!  \brief  allocate a new field geometry.  */
field_geometry *allocate_field_geometry(void);
/*!  \brief  free a field geometry.
     \param  geometry  field geometry to free  */
void free_field_geometry(field_geometry *geometry);
/*!  \brief  returns a nubmer of lattice points.
     \param  geometry  geometry of a field  */
int lattice_point_number(field_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
